@@ -5,7 +5,7 @@
  */
 
 mergeInto(LibraryManager.library, {
-  $FS__deps: ['$getRandomDevice', '$PATH', '$PATH_FS', '$TTY', '$MEMFS',
+  $FS__deps: ['$getRandomDevice', '$PATH', '$PATH_FS', '$TTY', '$MEMFS', '$asyncLoad',
 #if LibraryManager.has('library_idbfs.js')
     '$IDBFS',
 #endif
@@ -1876,7 +1876,7 @@ FS.staticInit();` +
       }
       addRunDependency(dep);
       if (typeof url == 'string') {
-        Browser.asyncLoad(url, function(byteArray) {
+        asyncLoad(url, function(byteArray) {
           processData(byteArray);
         }, onerror);
       } else {
@@ -1993,15 +1993,6 @@ FS.staticInit();` +
     },
 #endif
   },
-
-  // Allocate memory for an mmap operation. This allocates space of the right
-  // page-aligned size, and clears the padding.
-  $mmapAlloc: function(size) {
-    var alignedSize = alignMemory(size, {{{ WASM_PAGE_SIZE }}});
-    var ptr = {{{ makeMalloc('mmapAlloc', 'alignedSize') }}};
-    while (size < alignedSize) HEAP8[ptr + size++] = 0;
-    return ptr;
-  },
 });
 
 if (FORCE_FILESYSTEM) {
