@@ -1083,6 +1083,7 @@ function addAtInit(code) {
 global.ATMAINS = [];
 
 function addAtMain(code) {
+  assert(HAS_MAIN, 'addAtMain called but program has no main function');
   ATMAINS.push(code);
 }
 
@@ -1212,12 +1213,20 @@ function makeModuleReceiveWithVar(localName, moduleName, defaultValue, noAssert)
 }
 
 function makeRemovedFSAssert(fsName) {
-  if (!ASSERTIONS) return;
+  assert(ASSERTIONS);
   const lower = fsName.toLowerCase();
-  if (SYSTEM_JS_LIBRARIES.includes('library_' + lower + '.js')) return '';
+  if (JS_LIBRARIES.includes('library_' + lower + '.js')) return '';
   return `var ${fsName} = '${fsName} is no longer included by default; build with -l${lower}.js';`;
 }
 
+function makeRemovedRuntimeFunction(name) {
+  assert(ASSERTIONS);
+  if (libraryFunctions.includes(name)) {
+    return '';
+  }
+  return `function ${name}() { abort('\`${name}\` is now a library function and not included by default; add it to your library.js __deps or to DEFAULT_LIBRARY_FUNCS_TO_INCLUDE on the command line'); }`;
+}
+
 // Given an array of elements [elem1,elem2,elem3], returns a string "['elem1','elem2','elem3']"
 function buildStringArray(array) {
   if (array.length > 0) {
