@@ -37,7 +37,7 @@ var ENVIRONMENT_IS_SHELL = typeof read === 'function';
 var ENVIRONMENT_IS_WEB = true
 #elif ENVIRONMENT && !ENVIRONMENT.includes(',')
 var ENVIRONMENT_IS_WEB = {{{ ENVIRONMENT === 'web' }}};
-#elif ENVIRONMENT_MAY_BE_SHELL && ENVIRONMENT_IS_NODE
+#elif ENVIRONMENT_MAY_BE_SHELL && ENVIRONMENT_MAY_BE_NODE
 var ENVIRONMENT_IS_WEB = !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_SHELL;
 #elif ENVIRONMENT_MAY_BE_SHELL
 var ENVIRONMENT_IS_WEB = !ENVIRONMENT_IS_SHELL;
