@@ -4,9 +4,9 @@
 // found in the LICENSE file.
 
 #include <pthread.h>
-#include <emscripten.h>
-#include <emscripten/threading.h>
+#include <emscripten/em_asm.h>
 #include <assert.h>
+#include <stdio.h>
 #include <stdlib.h>
 
 volatile int result = 0;
@@ -15,32 +15,27 @@ static void *thread2_start(void *arg)
 {
   EM_ASM(out('thread2_start!'));
   ++result;
-
-  pthread_exit(0);
+  return NULL;
 }
 
 static void *thread1_start(void *arg)
 {
   EM_ASM(out('thread1_start!'));
   pthread_t thr;
-  pthread_create(&thr, NULL, thread2_start, 0);
-  pthread_join(thr, 0);
-  pthread_exit(0);
+  int rtn = pthread_create(&thr, NULL, thread2_start, NULL);
+#ifdef SMALL_POOL
+  assert(rtn != 0);
+#else
+  assert(rtn == 0);
+  pthread_join(thr, NULL);
+#endif
+  return NULL;
 }
 
 int main()
 {
-  if (!emscripten_has_threading_support())
-  {
-#ifdef REPORT_RESULT
-    REPORT_RESULT(1);
-#endif
-    printf("Skipped: Threading is not supported.\n");
-    return 0;
-  }
-
   pthread_t thr;
-  pthread_create(&thr, NULL, thread1_start, 0);
+  pthread_create(&thr, NULL, thread1_start, NULL);
 
   pthread_attr_t attr;
   pthread_getattr_np(thr, &attr);
@@ -48,12 +43,16 @@ int main()
   void *stack_addr;
   pthread_attr_getstack(&attr, &stack_addr, &stack_size);
   printf("stack_size: %d, stack_addr: %p\n", (int)stack_size, stack_addr);
-  if (stack_size != 2*1024*1024 || stack_addr == 0)
+  if (stack_size != 2*1024*1024 || stack_addr == NULL)
     result = -100; // Report failure.
 
-  pthread_join(thr, 0);
+  pthread_join(thr, NULL);
 
-#ifdef REPORT_RESULT
-  REPORT_RESULT(result);
+  printf("done result=%d\n", result);
+#ifdef SMALL_POOL
+  assert(result == 0);
+#else
+  assert(result == 1);
 #endif
+  return 0;
 }
