@@ -15,15 +15,7 @@ __rootpath__ = os.path.abspath(os.path.dirname(os.path.dirname(__file__)))
 sys.path.insert(1, __rootpath__)
 
 from tools.toolchain_profiler import ToolchainProfiler
-from tools import building, config
-if __name__ == '__main__':
-  ToolchainProfiler.record_process_start()
-
-try:
-  from tools import shared
-except ImportError:
-  # Python 2 circular import compatibility
-  import shared
+from tools import building, config, shared, utils
 
 configuration = shared.configuration
 temp_files = configuration.get_temp_files()
@@ -132,26 +124,26 @@ end_asm_marker = '// EMSCRIPTEN_END_ASM\n'
 
 # Given a set of functions of form (ident, text), and a preferred chunk size,
 # generates a set of chunks for parallel processing and caching.
+@ToolchainProfiler.profile_block('chunkify')
 def chunkify(funcs, chunk_size):
-  with ToolchainProfiler.profile_block('chunkify'):
-    chunks = []
-    # initialize reasonably, the rest of the funcs we need to split out
-    curr = []
-    total_size = 0
-    for i in range(len(funcs)):
-      func = funcs[i]
-      curr_size = len(func[1])
-      if total_size + curr_size < chunk_size:
-        curr.append(func)
-        total_size += curr_size
-      else:
-        chunks.append(curr)
-        curr = [func]
-        total_size = curr_size
-    if curr:
+  chunks = []
+  # initialize reasonably, the rest of the funcs we need to split out
+  curr = []
+  total_size = 0
+  for i in range(len(funcs)):
+    func = funcs[i]
+    curr_size = len(func[1])
+    if total_size + curr_size < chunk_size:
+      curr.append(func)
+      total_size += curr_size
+    else:
       chunks.append(curr)
-      curr = None
-    return [''.join(func[1] for func in chunk) for chunk in chunks] # remove function names
+      curr = [func]
+      total_size = curr_size
+  if curr:
+    chunks.append(curr)
+    curr = None
+  return [''.join(func[1] for func in chunk) for chunk in chunks] # remove function names
 
 
 def run_on_js(filename, passes, extra_info=None, just_split=False, just_concat=False):
@@ -159,7 +151,7 @@ def run_on_js(filename, passes, extra_info=None, just_split=False, just_concat=F
     if not isinstance(passes, list):
       passes = [passes]
 
-    js = open(filename).read()
+    js = utils.read_file(filename)
     if os.linesep != '\n':
       js = js.replace(os.linesep, '\n') # we assume \n in the splitting code
 
@@ -341,7 +333,7 @@ EMSCRIPTEN_FUNCS();
             acorn_passes.append('minifyWhitespace')
           cld = building.acorn_optimizer(cld, acorn_passes)
           temp_files.note(cld)
-        coutput = open(cld).read()
+        coutput = utils.read_file(cld)
 
       coutput = coutput.replace('wakaUnknownBefore();', start_asm)
       after = 'wakaUnknownAfter'
@@ -372,7 +364,7 @@ EMSCRIPTEN_FUNCS();
       # sort functions by size, to make diffing easier and to improve aot times
       funcses = []
       for out_file in filenames:
-        funcses.append(split_funcs(open(out_file).read(), False))
+        funcses.append(split_funcs(utils.read_file(out_file), False))
       funcs = [item for sublist in funcses for item in sublist]
       funcses = None
       if not os.environ.get('EMCC_NO_OPT_SORT'):
@@ -389,7 +381,7 @@ EMSCRIPTEN_FUNCS();
     else:
       # just concat the outputs
       for out_file in filenames:
-        f.write(open(out_file).read())
+        f.write(utils.read_file(out_file))
 
   with ToolchainProfiler.profile_block('write_post'):
     f.write('\n')
@@ -401,15 +393,14 @@ EMSCRIPTEN_FUNCS();
   return filename
 
 
+@ToolchainProfiler.profile_block('js_optimizer.run_on_js')
 def run(filename, passes, extra_info=None):
   just_split = 'receiveJSON' in passes
   just_concat = 'emitJSON' in passes
-  with ToolchainProfiler.profile_block('js_optimizer.run_on_js'):
-    return run_on_js(filename, passes, extra_info=extra_info, just_split=just_split, just_concat=just_concat)
+  return run_on_js(filename, passes, extra_info=extra_info, just_split=just_split, just_concat=just_concat)
 
 
 def main():
-  ToolchainProfiler.record_process_start()
   last = sys.argv[-1]
   if '{' in last:
     extra_info = json.loads(last)
