@@ -9,6 +9,8 @@ import logging
 TAG = 'version_2'
 HASH = '317b22ad9b6b2f7b40fac7b7c426da2fa2da1803bbe58d480631f1e5b190d730763f2768c77c72affa806c69a1e703f401b15a1be3ec611cd259950d5ebc3711'
 
+deps = ['sdl2']
+
 
 def needed(settings):
   return settings.USE_SDL_NET == 2
@@ -42,6 +44,10 @@ def clear(ports, settings, shared):
   shared.Cache.erase_lib('libSDL2_net.a')
 
 
+def process_dependencies(settings):
+  settings.USE_SDL = 2
+
+
 def process_args(ports):
   return []
 
